/*
* glue.c
* glue exe and script
* Luiz Henrique de Figueiredo <lhf@tecgraf.puc-rio.br>
* 26 Aug 2004 19:18:22
* This code is hereby placed in the public domain.
*/

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "glue.h"

# define IS_DIR_SEPARATOR(ch) (((ch) == '/') || ((ch) == '\\'))

char *basename(const char *name) {
   const char *base;

#if defined (HAVE_DOS_BASED_FILE_SYSTEM)
   /* Skip over the disk name in MSDOS pathnames. */
   if (isalpha (name[0]) && name[1] == ':') 
      name += 2;
#endif
   
   for (base = name; *name; name++) {
      if (IS_DIR_SEPARATOR (*name)) {
         base = name + 1;
      }
   }
   return (char *) base;
}

static void cannot(const char* what, const char* name)
{
 fprintf(stderr,"cannot %s %s: %s\n",what,name,strerror(errno));
 exit(EXIT_FAILURE);
}

static long copy(const char* name, FILE* out, const char* outname)
{
 FILE* f;
 long size;
 f=fopen(name,"rb");
 if (f==NULL) cannot("open",name);
 if (fseek(f,0,SEEK_END)!=0) cannot("seek",name);
 size=ftell(f);
 if (fseek(f,0,SEEK_SET)!=0) cannot("seek",name);
 for (;;)
 {
  char b[BUFSIZ];
  int n=fread(&b,1,sizeof(b),f);
  if (n==0) {
     if (ferror(f)) 
        cannot("read",name); 
     else 
        break;
  }
  if (fwrite(&b,n,1,out)!=1) cannot("write",outname);
 }
 fclose(f);
 /* fprintf(stderr, "wrote: %d\n", size); */
 return size;
}

int main(int argc, char* argv[])
{
 Glue *e;
 Glue tail;
 int i,n,dpos,npos;
 char *me, *header, *output, **files;
 FILE* f;
 if (argc<4)
 {
  fprintf(stderr,"usage: glue header.exe output.exe f1.lua f2.lua ...\n");
  return 1;
 }

 me     = argv[0];
 header = argv[1];
 output = argv[2];
 files  = &argv[3];
 n = argc - 3;
 /* Allocate entries and tail */
 e = malloc(n * sizeof(Glue));
 if (e == NULL) cannot("initialize", argv[0]);

 /* Get ready for writing */
 f=fopen(output,"wb");
 if (f==NULL) cannot("open",output);
 
 /* Copy header executable */
 dpos = copy(header, f, output);
 npos = n * sizeof(Glue);

 /* the files we want to add */
 files = &argv[3];
 for (i = 0; i < n; i++, files++) {
    /* segment starts */
    e[i].start = dpos;
    /* string start from head of glue */
    e[i].name = npos;
    /* fprintf(stderr, "name: %s start: %d npos: %d ", *files, (int)e[i].start, (int)e[i].name); */
    /* add file to output, update dataposition */
    dpos += copy(*files, f, output);
    /* just update string position, don't forget terminator */
    npos += strlen(basename(*files)) + 1;
 }

 /* write entries */
 for (i = 0; i < n; i++) {
    if (fwrite(&e[i], sizeof(Glue),1,f) != 1) cannot("write", output);
 }
 /* write names */
 files = &argv[3];
 for (i = 0; i < n; i++, files++) {
    const char *bname = basename(*files);
    /* fprintf(stderr, "storing name: %s\n", bname); */
    if (fwrite(bname, strlen(bname)+1,1,f) != 1) cannot("write", output);
 }

 /* Enough info to fill in tail */
 tail.start = dpos;
 tail.name = n;
 
 /* write tail */
 if (fwrite(&tail, sizeof(Glue),1,f) != 1) cannot("write", output);
 if (fwrite(GLUESIG, GLUELEN,1,f) != 1) cannot("write", output);

 if (fclose(f)!=0) cannot("close",argv[3]);
 free(e);
 return 0;
}
